﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;
using System.Linq;
using System.Reflection;


namespace TimeScheduler.Model.Impl
{
  class FileDomain : IDomain
  {
    #region Konstruktion
    private List<TimeItem> allItems_ = new List<TimeItem>();
    private string filename_ = string.Empty;

    public FileDomain()
    {
      var fi = new FileInfo(System.Reflection.Assembly.GetExecutingAssembly().Location);
      filename_ = Path.Combine(fi.DirectoryName, fi.Name + ".xmldata");

      if (File.Exists(filename_))
        Load();
      else
      {
        // Wenn nichts vorhanden, dann einen Spielsatz anlegen
        allItems_ = new List<TimeItem> {
          new TimeItem { Description = "Desc 01", From = DateTime.Now.Subtract(TimeSpan.FromHours(2)), Till = DateTime.Now, CostUnit = 14, ItemType = TimeItemType.Normal },
          new TimeItem { Description = "Desc 02", From = DateTime.Now.Subtract(TimeSpan.FromHours(2)), Till = DateTime.Now, CostUnit = 13, ItemType = TimeItemType.Break },
          new TimeItem { Description = "Desc 03", From = DateTime.Now.Subtract(TimeSpan.FromHours(2)), Till = DateTime.Now, CostUnit = 12, ItemType = TimeItemType.Holiday },
          new TimeItem { Description = "Desc 04", From = DateTime.Now.Subtract(TimeSpan.FromHours(2)), Till = DateTime.Now, CostUnit = 11, ItemType = TimeItemType.Ill },
          new TimeItem { Description = "Desc 05", From = DateTime.Now.Subtract(TimeSpan.FromHours(2)), Till = DateTime.Now, CostUnit = 15, ItemType = TimeItemType.Normal },
        };
      }
    }
    #endregion

    #region Hilfsfunktionen
    private void ReadList(string filePath)
    {
      var bin = new BinaryFormatter();
      using (var rd = new FileStream(filePath, FileMode.Open))
        allItems_ = bin.Deserialize(rd) as List<TimeItem>;
    }

    private void WriteList(string path)
    {
      var bin = new BinaryFormatter();
      using (var wr = new FileStream(path, FileMode.OpenOrCreate))
        bin.Serialize(wr, allItems_);
    }

    private void ReadListXml(string filePath)
    {
      var xs = new XmlSerializer(typeof(List<TimeItem>));
      using (var rd = new StreamReader(filePath))
        allItems_ = xs.Deserialize(rd) as List<TimeItem>;
    }
    private void WriteListXml(string path)
    {
      var xs = new XmlSerializer(typeof(List<TimeItem>));
      using (StreamWriter wr = new StreamWriter(path))
        xs.Serialize(wr, allItems_);
    }

    private void Load()
    {
      ReadListXml(filename_);
    }

    private void Save()
    {
      var newFilename = filename_ + ".new";
      WriteListXml(newFilename);
      File.Delete(filename_);
      File.Move(newFilename, filename_);
    }
    #endregion

    #region IDomain
    public List<ITimeItem> GetItems(DateTime date)
    {
      var founded = allItems_.Where(x => x.From.Date == date);
      return founded.Cast<ITimeItem>().ToList();
    }

    public Dictionary<int, string> GetCostUnits()
    {
      return new Dictionary<int, string>
      {
        { 11, "Allgemein CC1" },
        { 12, "Allgemein CC2" },
        { 13, "Allgemein CC3" },
        { 14, "Allgemein IC4" },
        { 15, "Allgemein CC5" },
        { 16, "Allgemein CC6" },
        { 17, "Allgemein CC7" },
      };
    }

    public void ElementAdded(ITimeItem timeItem)
    {
      allItems_.Add((TimeItem)timeItem);
      Save();
    }
    public void ElementChanged(ITimeItem timeItem)
    {
      allItems_.Remove((TimeItem)timeItem);
      allItems_.Add((TimeItem)timeItem);
      Save();
    }
    public void ElementRemoved(ITimeItem timeItem)
    {
      allItems_.Remove((TimeItem)timeItem);
      Save();
    }
    #endregion
  }
}
